#include "trtSettings.h"
#include "trtkernel_1284.c"
#include "trtTimers.c"
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>

// serial communication library
// Don't mess with the semaphores
#define SEM_RX_ISR_SIGNAL 1
#define SEM_STRING_DONE 2
#include "trtUart.h"
#include "trtUart.c"

#define SEM_SHARED 3

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// Variables
int xcoord, ycoord, screenX, screenY, xcoord_old, ycoord_old, delta_y;
volatile uint8_t leftclick, rightclick, scroll;
int scrolldir;

// the usual
#define begin {
#define end }

// input arguments to each thread
// not actually used in this example
int args[3];

// ------ define task 1 -----------
void pccomm()
begin
	uint32_t rel, dead;
	int x, y;
	float tmpx, tmpy;

	while (1) {
		// save the old screen values
		ycoord_old = ycoord;
		
		// receive raw coordinates from Matlab
		fscanf(stdin, " %d %d", &x, &y);
		
		// scale the resolution of the coordinates for screen
		tmpx = (1280.0/620.0)*x;
		tmpy = (1024.0/460.0)*y;
		xcoord = (int) tmpx;
		ycoord = (int) tmpy;


		// determine direction of scroll
		delta_y = ycoord - ycoord_old;

		if(scroll) {
			if (delta_y >= 10)
				scrolldir = 1;
			else if (delta_y <= -10)
				scrolldir = -1;
		}
		else
			scrolldir = 0;

		// check switch and encapsulate variables in packet and send to Matlab
		trtWait(SEM_SHARED);
		if (~PINB & (1<<PINB0)) {
			fprintf(stdout,"%d %d %d %d %d %d",xcoord,ycoord, 1, leftclick, rightclick, scrolldir);
		}
		else
			fprintf(stdout,"%d %d %d %d %d %d",xcoord,ycoord, 0, leftclick, rightclick, scrolldir);
		trtSignal(SEM_SHARED);
		
		rel = trtCurrentTime() + SECONDS2TICKS(0.001);
	    dead = trtCurrentTime() + SECONDS2TICKS(0.05);
	    trtSleepUntil(rel, dead);
	}
end
 
// -------- define task 2 ---------
void buttoncomm()
begin
	uint8_t state;
	uint32_t rel, dead ;

	state = 0;

	while(1) {
		// button comm FSM
		trtWait(SEM_SHARED);
		switch (state)
		begin
			// listen for button presses
			case 0:
			begin
				// left click
				if (~PINB & (1<<PINB1)) {
					leftclick = 1;
					state = 1;
				}
				// right click
				else if (~PINB & (1<<PINB3)) {
					rightclick = 1;
					state = 2;
				}
				// scroll
				else if (~PINB & (1<<PINB2)) {
					scroll = 1;
					state = 3;
				}
				break;
			end

			// left click button pressed
			case 1:
			begin
				if (PINB & (1<<PINB1)) {
					leftclick = 0;
					state = 0;
				}
				break;
			end

			// right click button pressed
			case 2:
			begin
				if (PINB & (1<<PINB3)) {
					rightclick = 0;
					state = 0;
				}
				break;
				
			end

			// scroll button pressed
			case 3:
			begin
				if (PINB & (1<<PINB2)) {
					scroll = 0;
					state = 0;
				}
				break;
			end
		end
		trtSignal(SEM_SHARED);

		rel = trtCurrentTime() + SECONDS2TICKS(0.001);
		dead = trtCurrentTime() + SECONDS2TICKS(0.002);
		trtSleepUntil(rel, dead);
	}
end

// --- Main Program ----------------------------------
int main(void) {
	DDRB = 0x00;	// set Port B to inputs
	PORTB = 0xff;	// set pull-up resistors

	// start TRT
	trtInitKernel(200); // 200 bytes for the idle task stack

	// init the UART -- trt_uart_init() is in trtUart.c
	trt_uart_init();
	stdout = stdin = stderr = &uart_str;

	// set original x- and y- coordinates to 0
	xcoord = 0;
	ycoord = 0;

	// =========================================
	// SET RESOLUTION OF COMPUTER SCREEN HERE
	// =========================================
	screenX = 1280;
	screenY = 1024;

	// --- create semaphores ----------
	// You must creat the first two semaphores if you use the uart
	trtCreateSemaphore(SEM_RX_ISR_SIGNAL, 0) ; // uart receive ISR semaphore
	trtCreateSemaphore(SEM_STRING_DONE,0) ;  // user typed <enter>
	trtCreateSemaphore(SEM_SHARED,1);

	// --- create tasks  ----------------
	trtCreateTask(pccomm, 800, SECONDS2TICKS(0.1), SECONDS2TICKS(0.3), &(args[0]));
	trtCreateTask(buttoncomm, 400, SECONDS2TICKS(0.1), SECONDS2TICKS(0.3), &(args[0]));

	// --- Idle task --------------------------------------
	// just sleeps the cpu to save power 
	// every time it executes
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_enable();
	while (1) 
	begin
		//sleep_cpu();
	end

} // main
